=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-02-06 07:55:42 +0000
+++ bzrlib/errors.py	2006-02-20 22:05:28 +0000
@@ -180,6 +180,10 @@
     """File exists: %(path)r%(extra)s"""
 
 
+class DirectoryNotEmpty(PathError):
+    """Directory not empty: %(path)r%(extra)s"""
+
+
 class PermissionDenied(PathError):
     """Permission denied: %(path)r%(extra)s"""
 

=== modified file 'bzrlib/tests/test_transport_implementations.py'
--- bzrlib/tests/test_transport_implementations.py	2006-01-24 04:44:33 +0000
+++ bzrlib/tests/test_transport_implementations.py	2006-02-20 22:05:28 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005 by Canonical Ltd
+# Copyright (C) 2004, 2005, 2006 by Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,8 +25,9 @@
 import stat
 import sys
 
-from bzrlib.errors import (NoSuchFile, FileExists,
+from bzrlib.errors import (DirectoryNotEmpty, NoSuchFile, FileExists,
                            LockError,
+                           PathError,
                            TransportNotPossible, ConnectionError)
 from bzrlib.tests import TestCaseInTempDir, TestSkipped
 from bzrlib.transport import memory, urlescape
@@ -552,6 +553,19 @@
         t.rmdir('adir')
         self.assertRaises(NoSuchFile, t.stat, 'adir')
 
+    def test_rmdir_not_empty(self):
+        """Deleting a non-empty directory raises an exception
+        
+        sftp (and possibly others) don't give us a specific "directory not
+        empty" exception -- we can just see that the operation failed.
+        """
+        t = self.get_transport()
+        if t.is_readonly():
+            return
+        t.mkdir('adir')
+        t.mkdir('adir/bdir')
+        self.assertRaises(PathError, t.rmdir, 'adir')
+
     def test_delete_tree(self):
         t = self.get_transport()
 

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2006-02-06 07:55:42 +0000
+++ bzrlib/transport/__init__.py	2006-02-20 22:05:28 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006 Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -107,6 +107,8 @@
                 raise errors.FileExists(path, extra=e)
             if e.errno == errno.EACCES:
                 raise errors.PermissionDenied(path, extra=e)
+            if e.errno == errno.ENOTEMPTY:
+                raise errors.DirectoryNotEmpty(path, extra=e)
         if raise_generic:
             raise errors.TransportError(orig_error=e)
 

=== modified file 'bzrlib/transport/memory.py'
--- bzrlib/transport/memory.py	2006-01-24 04:44:33 +0000
+++ bzrlib/transport/memory.py	2006-02-20 22:05:28 +0000
@@ -162,7 +162,7 @@
                 self._translate_error(IOError(errno.EBUSY, relpath), relpath)
         for path in self._dirs:
             if path.startswith(_abspath) and path != _abspath:
-                self._translate_error(IOError(errno.EBUSY, relpath), relpath)
+                self._translate_error(IOError(errno.ENOTEMPTY, relpath), relpath)
         if not _abspath in self._dirs:
             raise NoSuchFile(relpath)
         del self._dirs[_abspath]

